@@ -27,9 +27,12 @@ static PyObject* _Nullable call_NSCoder_encodeValueOfObjCType_at_(
     value = arguments[1];
 
     size = PyObjCRT_SizeOfType(view.buf);
-    if (size == -1) {
+    if (size == -1) { // LCOV_BR_EXCL_LINE
+        /* Value has already been validated */
+        // LCOV_EXCL_START
         PyBuffer_Release(&view);
         return NULL;
+        // LCOV_EXCL_STOP
     }
 
     buf = PyMem_Malloc(size);
@@ -57,15 +60,15 @@ static PyObject* _Nullable call_NSCoder_encodeValueOfObjCType_at_(
                     buf);
 
             } else {
-                super.receiver    = PyObjCObject_GetObject(self);
+                super.receiver    = (id _Nonnull)PyObjCObject_GetObject(self);
                 super.super_class = PyObjCSelector_GetClass(method);
 
                 ((void (*)(struct objc_super*, SEL, char*, void*))objc_msgSendSuper)(
                     &super, PyObjCSelector_GetSelector(method), view.buf, buf);
             }
 
-        } @catch (NSObject* localException) {
-            PyObjCErr_FromObjC(localException);
+        } @catch (NSObject* localException) { // LCOV_EXCL_LINE
+            PyObjCErr_FromObjC(localException); // LCOV_EXCL_LINE
         }
     Py_END_ALLOW_THREADS
 
@@ -76,8 +79,7 @@ static PyObject* _Nullable call_NSCoder_encodeValueOfObjCType_at_(
         return NULL;
     }
 
-    Py_INCREF(Py_None);
-    return Py_None;
+    Py_RETURN_NONE;
 }
 
 static IMP
@@ -181,10 +183,12 @@ static PyObject* _Nullable call_NSCoder_encodeArrayOfObjCType_count_at_(
     }
 
     buf = PyMem_Malloc(size * (count + 1));
-    if (buf == NULL) {
+    if (buf == NULL) { // LCOV_BR_EXCL_LINE
+        // LCOV_EXCL_START
         PyBuffer_Release(&view);
         PyErr_NoMemory();
         return NULL;
+        // LCOV_EXCL_STOP
     }
 
     if (!PySequence_Check(value)) {
@@ -226,7 +230,7 @@ static PyObject* _Nullable call_NSCoder_encodeArrayOfObjCType_count_at_(
                     count, buf);
             } else {
                 super.super_class = PyObjCSelector_GetClass(method);
-                super.receiver    = PyObjCObject_GetObject(self);
+                super.receiver    = (id _Nonnull)PyObjCObject_GetObject(self);
 
                 ((void (*)(struct objc_super*, SEL, char*, NSUInteger,
                            void*))objc_msgSendSuper)(
@@ -244,8 +248,7 @@ static PyObject* _Nullable call_NSCoder_encodeArrayOfObjCType_count_at_(
     if (PyErr_Occurred())
         return NULL;
 
-    Py_INCREF(Py_None);
-    return Py_None;
+    Py_RETURN_NONE;
 }
 
 static IMP
@@ -366,10 +369,12 @@ static PyObject* _Nullable call_NSCoder_decodeValueOfObjCType_at_(
         return NULL;
     }
     buf = PyMem_Malloc(size);
-    if (buf == NULL) {
+    if (buf == NULL) { // LCOV_BR_EXCL_LINE
+        // LCOV_EXCL_START
         PyBuffer_Release(&view);
         PyErr_NoMemory();
         return NULL;
+        // LCOV_EXCL_STOP
     }
 
     int isIMP = PyObjCIMP_Check(method);
@@ -381,7 +386,7 @@ static PyObject* _Nullable call_NSCoder_decodeValueOfObjCType_at_(
                     buf);
             } else {
                 super.super_class = PyObjCSelector_GetClass(method);
-                super.receiver    = PyObjCObject_GetObject(self);
+                super.receiver    = (id _Nonnull)PyObjCObject_GetObject(self);
 
                 ((void (*)(struct objc_super*, SEL, char*, void*))objc_msgSendSuper)(
                     &super, PyObjCSelector_GetSelector(method), view.buf, buf);
@@ -495,10 +500,12 @@ static PyObject* _Nullable call_NSCoder_decodeValueOfObjCType_at_size_(
     }
 
     buf = PyMem_Malloc(size);
-    if (buf == NULL) {
+    if (buf == NULL) { // LCOV_BR_EXCL_LINE
+        // LCOV_EXCL_START
         PyBuffer_Release(&view);
         PyErr_NoMemory();
         return NULL;
+        // LCOV_EXCL_STOP
     }
 
     int isIMP = PyObjCIMP_Check(method);
@@ -510,7 +517,7 @@ static PyObject* _Nullable call_NSCoder_decodeValueOfObjCType_at_size_(
                     buf, size);
             } else {
                 super.super_class = PyObjCSelector_GetClass(method);
-                super.receiver    = PyObjCObject_GetObject(self);
+                super.receiver    = (id _Nonnull)PyObjCObject_GetObject(self);
 
                 ((void (*)(struct objc_super*, SEL, char*, void*,
                            NSUInteger))objc_msgSendSuper)(
@@ -643,10 +650,12 @@ static PyObject* _Nullable call_NSCoder_decodeArrayOfObjCType_count_at_(
     }
 
     buf = PyMem_Malloc(size * (count + 1));
-    if (buf == NULL) {
+    if (buf == NULL) { // LCOV_BR_EXCL_LINE
+        // LCOV_EXCL_START
         PyBuffer_Release(&view);
         PyErr_NoMemory();
         return NULL;
+        // LCOV_EXCL_STOP
     }
 
     int isIMP = PyObjCIMP_Check(method);
@@ -658,7 +667,7 @@ static PyObject* _Nullable call_NSCoder_decodeArrayOfObjCType_count_at_(
                     count, buf);
             } else {
                 super.super_class = PyObjCSelector_GetClass(method);
-                super.receiver    = PyObjCObject_GetObject(self);
+                super.receiver    = (id _Nonnull)PyObjCObject_GetObject(self);
 
                 ((void (*)(struct objc_super*, SEL, char*, NSUInteger,
                            void*))objc_msgSendSuper)(&super,
@@ -750,18 +759,18 @@ mkimp_NSCoder_decodeArrayOfObjCType_count_at_(PyObject*              callable,
               return;
           }
 
-          seq = PySequence_Fast(result, "Return-value must be a sequence");
+          seq = PyObjCSequence_Tuple(result, "Return-value must be a sequence");
           Py_DECREF(result);
           if (seq == NULL)
               goto error;
 
-          if ((NSUInteger)PySequence_Fast_GET_SIZE(seq) != count) {
+          if ((NSUInteger)PyTuple_GET_SIZE(seq) != count) {
               PyErr_SetString(PyExc_TypeError, "return value must be a of correct size");
               goto error;
           }
 
           for (i = 0; i < count; i++) {
-              res = depythonify_c_value(typestr, PySequence_Fast_GET_ITEM(seq, i),
+              res = depythonify_c_value(typestr, PyTuple_GET_ITEM(seq, i),
                                         ((char*)buf) + (i * size));
               if (res == -1)
                   goto error;
@@ -820,7 +829,7 @@ static PyObject* _Nullable call_NSCoder_encodeBytes_length_(PyObject*        met
                     length);
             } else {
                 super.super_class = PyObjCSelector_GetClass(method);
-                super.receiver    = PyObjCObject_GetObject(self);
+                super.receiver    = (id _Nonnull)PyObjCObject_GetObject(self);
 
                 ((void (*)(struct objc_super*, SEL, void*, NSUInteger))objc_msgSendSuper)(
                     &super, PyObjCSelector_GetSelector(method), view.buf, length);
@@ -835,8 +844,7 @@ static PyObject* _Nullable call_NSCoder_encodeBytes_length_(PyObject*        met
     if (PyErr_Occurred())
         return NULL;
 
-    Py_INCREF(Py_None);
-    return Py_None;
+    Py_RETURN_NONE;
 }
 
 static IMP
@@ -927,7 +935,7 @@ static PyObject* _Nullable call_NSCoder_decodeBytesWithReturnedLength_(
                     PyObjCObject_GetObject(self), PyObjCIMP_GetSelector(method), &size);
             } else {
                 super.super_class = PyObjCSelector_GetClass(method);
-                super.receiver    = PyObjCObject_GetObject(self);
+                super.receiver    = (id _Nonnull)PyObjCObject_GetObject(self);
 
                 bytes =
                     ((void* (*)(struct objc_super*, SEL, NSUInteger*))objc_msgSendSuper)(
@@ -1072,7 +1080,7 @@ static PyObject* _Nullable call_NSCoder_decodeBytesForKey_returnedLength_(
                     (NSUInteger*)&size);
             } else {
                 super.super_class = PyObjCSelector_GetClass(method);
-                super.receiver    = PyObjCObject_GetObject(self);
+                super.receiver    = (id _Nonnull)PyObjCObject_GetObject(self);
 
                 bytes = ((void* (*)(struct objc_super*, SEL, id,
                                     NSUInteger*))objc_msgSendSuper)(
@@ -1257,7 +1265,7 @@ static PyObject* _Nullable call_NSCoder_encodeBytes_length_forKey_(
                     (NSUInteger)view.len, key);
             } else {
                 super.super_class = PyObjCSelector_GetClass(method);
-                super.receiver    = PyObjCObject_GetObject(self);
+                super.receiver    = (id _Nonnull)PyObjCObject_GetObject(self);
 
                 ((void (*)(struct objc_super*, SEL, void*, NSUInteger,
                            id))objc_msgSendSuper)(&super,
@@ -1275,8 +1283,7 @@ static PyObject* _Nullable call_NSCoder_encodeBytes_length_forKey_(
     if (PyErr_Occurred()) {
         return NULL;
     }
-    Py_INCREF(Py_None);
-    return Py_None;
+    Py_RETURN_NONE;
 }
 
 static IMP
@@ -1353,93 +1360,100 @@ PyObjC_setup_nscoder(PyObject* module __attribute__((__unused__)))
 {
     Class classNSCoder = objc_lookUpClass("NSCoder");
 
-    if (PyObjC_RegisterMethodMapping(classNSCoder,
+    if (PyObjC_RegisterMethodMapping(classNSCoder, // LCOV_BR_EXCL_LINE
                                      @selector(encodeArrayOfObjCType:count:at:),
                                      call_NSCoder_encodeArrayOfObjCType_count_at_,
                                      mkimp_NSCoder_encodeArrayOfObjCType_count_at_)
         < 0) {
-        return -1;
+        return -1; // LCOV_EXCL_LINE
     }
 
-    if (PyObjC_RegisterMethodMapping(classNSCoder, @selector(encodeValueOfObjCType:at:),
+    if (PyObjC_RegisterMethodMapping(classNSCoder, // LCOV_BR_EXCL_LINE
+                                    @selector(encodeValueOfObjCType:at:),
                                      call_NSCoder_encodeValueOfObjCType_at_,
                                      mkimp_NSCoder_encodeValueOfObjCType_at_)
         < 0) {
-        return -1;
+        return -1; // LCOV_EXCL_LINE
     }
 
-    if (PyObjC_RegisterMethodMapping(classNSCoder,
+    if (PyObjC_RegisterMethodMapping(classNSCoder, // LCOV_BR_EXCL_LINE
                                      @selector(decodeArrayOfObjCType:count:at:),
                                      call_NSCoder_decodeArrayOfObjCType_count_at_,
                                      mkimp_NSCoder_decodeArrayOfObjCType_count_at_)
         < 0) {
-        return -1;
+        return -1; // LCOV_EXCL_LINE
     }
 
-    if (PyObjC_RegisterMethodMapping(classNSCoder, @selector(decodeValueOfObjCType:at:),
+    if (PyObjC_RegisterMethodMapping(classNSCoder, // LCOV_BR_EXCL_LINE
+                                    @selector(decodeValueOfObjCType:at:),
                                      call_NSCoder_decodeValueOfObjCType_at_,
                                      mkimp_NSCoder_decodeValueOfObjCType_at_)
         < 0) {
-        return -1;
+        return -1; // LCOV_EXCL_LINE
     }
 
-    if (PyObjC_RegisterMethodMapping(classNSCoder,
+    if (PyObjC_RegisterMethodMapping(classNSCoder, // LCOV_BR_EXCL_LINE
                                      @selector(decodeValueOfObjCType:at:size:),
                                      call_NSCoder_decodeValueOfObjCType_at_size_,
                                      mkimp_NSCoder_decodeValueOfObjCType_at_size_)
         < 0) {
-        return -1;
+        return -1; // LCOV_EXCL_LINE
     }
 
-    if (PyObjC_RegisterMethodMapping(classNSCoder, @selector(encodeBytes:length:),
+    if (PyObjC_RegisterMethodMapping(classNSCoder, // LCOV_BR_EXCL_LINE
+                                    @selector(encodeBytes:length:),
                                      call_NSCoder_encodeBytes_length_,
                                      mkimp_NSCoder_encodeBytes_length_)
         < 0) {
-        return -1;
+        return -1; // LCOV_EXCL_LINE
     }
 
-    if (PyObjC_RegisterMethodMapping(classNSCoder, @selector(encodeBytes:length:forKey:),
+    if (PyObjC_RegisterMethodMapping(classNSCoder, // LCOV_BR_EXCL_LINE
+                                    @selector(encodeBytes:length:forKey:),
                                      call_NSCoder_encodeBytes_length_forKey_,
                                      mkimp_NSCoder_encodeBytes_length_forKey_)
         < 0) {
-        return -1;
+        return -1; // LCOV_EXCL_LINE
     }
 
-    if (PyObjC_RegisterMethodMapping(classNSCoder,
+    if (PyObjC_RegisterMethodMapping(classNSCoder, // LCOV_BR_EXCL_LINE
                                      @selector(decodeBytesWithReturnedLength:),
                                      call_NSCoder_decodeBytesWithReturnedLength_,
                                      mkimp_NSCoder_decodeBytesWithReturnedLength_)
         < 0) {
-        return -1;
+        return -1; // LCOV_EXCL_LINE
     }
 
-    if (PyObjC_RegisterMethodMapping(classNSCoder,
+    if (PyObjC_RegisterMethodMapping(classNSCoder, // LCOV_BR_EXCL_LINE
                                      @selector(decodeBytesForKey:returnedLength::),
                                      call_NSCoder_decodeBytesForKey_returnedLength_,
                                      mkimp_NSCoder_decodeBytesForKey_returnedLength_)
         < 0) {
-        return -1;
+        return -1; // LCOV_EXCL_LINE
     }
 
-    if (PyObjC_RegisterMethodMapping(
-            classNSCoder, @selector(decodeBytesWithoutReturnedLength),
-            PyObjCUnsupportedMethod_Caller, PyObjCUnsupportedMethod_IMP)
+    if (PyObjC_RegisterMethodMapping(classNSCoder, // LCOV_BR_EXCL_LINE
+                                    @selector(decodeBytesWithoutReturnedLength),
+                                    PyObjCUnsupportedMethod_Caller,
+                                    PyObjCUnsupportedMethod_IMP)
         < 0) {
-        return -1;
+        return -1; // LCOV_EXCL_LINE
     }
 
-    if (PyObjC_RegisterMethodMapping(classNSCoder, @selector(encodeValuesOfObjCTypes:),
+    if (PyObjC_RegisterMethodMapping(classNSCoder, // LCOV_BR_EXCL_LINE
+                                    @selector(encodeValuesOfObjCTypes:),
                                      PyObjCUnsupportedMethod_Caller,
                                      PyObjCUnsupportedMethod_IMP)
         < 0) {
-        return -1;
+        return -1; // LCOV_EXCL_LINE
     }
 
-    if (PyObjC_RegisterMethodMapping(classNSCoder, @selector(decodeValuesOfObjCTypes:),
+    if (PyObjC_RegisterMethodMapping(classNSCoder, // LCOV_BR_EXCL_LINE
+                                    @selector(decodeValuesOfObjCTypes:),
                                      PyObjCUnsupportedMethod_Caller,
                                      PyObjCUnsupportedMethod_IMP)
         < 0) {
-        return -1;
+        return -1; // LCOV_EXCL_LINE
     }
 
     return 0;
