@@ -220,24 +220,29 @@ PyObjC_number_to_decimal(PyObject* pyValue, NSDecimal* outResult)
 
     if (PyLong_Check(pyValue)) {
         mantissa = PyLong_AsUnsignedLongLong(pyValue);
-        if (PyErr_Occurred()) {
+        if (mantissa == (unsigned long long)-1 && PyErr_Occurred()) {
             long long lng;
             PyErr_Clear();
             lng = PyLong_AsLongLong(pyValue);
 
-            if (PyErr_Occurred()) {
+            if (lng == -1 && PyErr_Occurred()) {
                 return -1;
             }
 
-            if (lng < 0) {
+            if (lng < 0) { // LCOV_BR_EXCL_LINE
                 mantissa = -lng;
                 exponent = 0;
                 negative = YES;
 
             } else {
+                /* Should never get here, `PyLong_AsUnsignedLongLong`
+                 * would not have failed for positive numbers.
+                 */
+                // LCOV_EXCL_START
                 mantissa = lng;
                 exponent = 0;
                 negative = NO;
+                // LCOV_EXCL_STOP
             }
 
             DecimalFromComponents(outResult, mantissa, exponent, negative);
@@ -273,9 +278,11 @@ PyObjC_number_to_decimal(PyObject* pyValue, NSDecimal* outResult)
                                                      __DECIMAL_DIG__,
 #endif
                                                      PyFloat_AsDouble(pyValue)];
-        if (stringVal == nil) {
+        if (stringVal == nil) { // LCOV_BR_EXCL_LINE
+            // LCOV_EXCL_START
             PyErr_SetString(PyObjCExc_Error, "Converting double to NSString failed");
             return -1;
+            // LCOV_EXCL_STOP
         }
 
         @try {
@@ -301,7 +308,7 @@ PyObjC_number_to_decimal(PyObject* pyValue, NSDecimal* outResult)
         _NSDecimalNumber_Class = PyObjCClass_New([NSDecimalNumber class]);
         if (_NSDecimalNumber_Class == NULL) { // LCOV_BR_EXCL_LINE
             PyErr_Clear();                    // LCOV_EXCL_LINE
-        }
+        } // LCOV_EXCL_LINE
     }
 
     if (_NSDecimalNumber_Class != NULL
@@ -345,26 +352,7 @@ decimal_init(PyObject* self, PyObject* _Nullable args, PyObject* _Nullable kwds)
             return -1;
         }
 
-        if (PyObjCObject_Check(pyValue)) {
-            NSObject* value;
-
-            if (depythonify_python_object(pyValue, &value) == -1) {
-                return -1;
-            }
-
-            if ([value isKindOfClass:[NSDecimalNumber class]]) {
-                ((DecimalObject*)self)->value = [(NSDecimalNumber*)value decimalValue];
-
-                ((DecimalObject*)self)->objc_value = (NSDecimalNumber*)value;
-                [value retain];
-                return 0;
-            }
-
-            PyErr_Format(PyExc_TypeError, "cannot convert instance of %s to NSDecimal",
-                         pyValue->ob_type->tp_name);
-            return -1;
-
-        } else if (PyUnicode_Check(pyValue)) {
+        if (PyUnicode_Check(pyValue)) {
 
             NSString* stringVal;
 
@@ -432,13 +420,13 @@ decimal_richcompare(PyObject* self, PyObject* other, int type)
             if (PyErr_Occurred()) {
                 PyErr_Clear();
             }
-            return PyBool_FromLong(0);
+            Py_RETURN_FALSE;
 
         } else if (type == Py_NE) {
             if (PyErr_Occurred()) {
                 PyErr_Clear();
             }
-            return PyBool_FromLong(1);
+            Py_RETURN_TRUE;
         }
 
         PyErr_Format(PyExc_TypeError, "Cannot compare NSDecimal and %s",
@@ -454,17 +442,17 @@ decimal_richcompare(PyObject* self, PyObject* other, int type)
 
     switch (type) {
     case Py_LT:
-        return PyBool_FromLong(res == NSOrderedAscending);
+        if(res == NSOrderedAscending) { Py_RETURN_TRUE; } else { Py_RETURN_FALSE; };
     case Py_LE:
-        return PyBool_FromLong(res != NSOrderedDescending);
+        if(res != NSOrderedDescending) { Py_RETURN_TRUE; } else { Py_RETURN_FALSE; };
     case Py_EQ:
-        return PyBool_FromLong(res == NSOrderedSame);
+        if(res == NSOrderedSame) { Py_RETURN_TRUE; } else { Py_RETURN_FALSE; };
     case Py_NE:
-        return PyBool_FromLong(res != NSOrderedSame);
+        if(res != NSOrderedSame) { Py_RETURN_TRUE; } else { Py_RETURN_FALSE; };
     case Py_GE:
-        return PyBool_FromLong(res != NSOrderedAscending);
+        if(res != NSOrderedAscending) { Py_RETURN_TRUE; } else { Py_RETURN_FALSE; };
     case Py_GT:
-        return PyBool_FromLong(res == NSOrderedDescending);
+        if(res == NSOrderedDescending) { Py_RETURN_TRUE; } else { Py_RETURN_FALSE; };
     default:
         // LCOV_EXCL_START
         PyErr_SetString(PyExc_TypeError, "Bad comparison arg");
@@ -532,8 +520,7 @@ static PyObject* _Nullable decimal_result_to_python(NSCalculationError status,
 #define TRY_COERCE(left, right)                                                          \
     int r = decimal_coerce(&left, &right);                                               \
     if (r == 1) {                                                                        \
-        Py_INCREF(Py_NotImplemented);                                                    \
-        return Py_NotImplemented;                                                        \
+        Py_RETURN_NOTIMPLEMENTED;                                                        \
     }
 
 #define DECIMAL_OPERATOR(py_function, nsdecimal_function)                                \
@@ -658,7 +645,7 @@ decimal_coerce(PyObject** l, PyObject** r)
         if (left == NULL) // LCOV_BR_EXCL_LINE
             goto error;   // LCOV_EXCL_LINE
 
-        args = Py_BuildValue("(O)", *l); /* XXX: really? */
+        args = PyTuple_Pack(1, *l);
         if (args == NULL)                // LCOV_BR_EXCL_LINE
             goto error;                  // LCOV_EXCL_LINE
 
@@ -679,7 +666,7 @@ decimal_coerce(PyObject** l, PyObject** r)
         if (right == NULL) // LCOV_BR_EXCL_LINE
             goto error;    // LCOV_EXCL_LINE
 
-        args = Py_BuildValue("(O)", *r); /* XXX: really? */
+        args = PyTuple_Pack(1, *r);
         if (args == NULL)                // LCOV_BR_EXCL_LINE
             goto error;                  // LCOV_EXCL_LINE
 
@@ -822,8 +809,8 @@ static PyObject* _Nullable call_NSDecimalNumber_decimalNumberWithDecimal_(
 
     Py_BEGIN_ALLOW_THREADS
         @try {
-            super.super_class = object_getClass(PyObjCSelector_GetClass(method));
-            super.receiver    = object_getClass(PyObjCClass_GetClass(self));
+            super.super_class = (Class _Nonnull)object_getClass(PyObjCSelector_GetClass(method));
+            super.receiver    = (id _Nonnull)object_getClass(PyObjCClass_GetClass(self));
 
             res = ((id(*)(struct objc_super*, SEL, NSDecimal))objc_msgSendSuper)(
                 &super, PyObjCSelector_GetSelector(method), *aDecimal);
@@ -864,7 +851,7 @@ static PyObject* _Nullable call_NSDecimalNumber_initWithDecimal_(
     Py_BEGIN_ALLOW_THREADS
         @try {
             super.super_class = PyObjCSelector_GetClass(method);
-            super.receiver    = PyObjCObject_GetObject(self);
+            super.receiver    = (id _Nonnull)PyObjCObject_GetObject(self);
 
             res = ((id(*)(struct objc_super*, SEL, NSDecimal))objc_msgSendSuper)(
                 &super, PyObjCSelector_GetSelector(method), *aDecimal);
@@ -952,7 +939,7 @@ static PyObject* _Nullable call_NSDecimalNumber_decimalValue(PyObject*        me
     Py_BEGIN_ALLOW_THREADS
         @try {
             super.super_class = PyObjCSelector_GetClass(method);
-            super.receiver    = PyObjCObject_GetObject(self);
+            super.receiver    = (id _Nonnull)PyObjCObject_GetObject(self);
 
 #if defined(__arm64__)
             aDecimal = ((NSDecimal(*)(struct objc_super*, SEL))objc_msgSendSuper)(
@@ -1089,8 +1076,9 @@ PyObjC_setup_nsdecimal(PyObject* m)
 
     if (@encode(NSDecimal)[1] == '?') {
         Decimal_Encoding[0] = '{';
-        strcpy(Decimal_Encoding + 1, "_NSDecimal");
-        strcpy(Decimal_Encoding + 11, @encode(NSDecimal) + 2);
+        Decimal_Encoding[1] = '\0';
+        strlcat(Decimal_Encoding, "_NSDecimal", sizeof(Decimal_Encoding));
+        strlcat(Decimal_Encoding, @encode(NSDecimal) + 2, sizeof(Decimal_Encoding));
         Decimal_Encoding_Len = strlen(Decimal_Encoding);
     }
 
@@ -1106,7 +1094,7 @@ PyObjC_setup_nsdecimal(PyObject* m)
 
     Class classNSDecimalNumberPlaceholder =
         objc_lookUpClass("NSDecimalNumberPlaceholder");
-    if (classNSDecimalNumberPlaceholder != nil) { // LCOV_BR_EXCL_LINE
+    if (classNSDecimalNumberPlaceholder != Nil) { // LCOV_BR_EXCL_LINE
         if (PyObjC_RegisterMethodMapping(classNSDecimalNumberPlaceholder,
                                          @selector(initWithDecimal:),
                                          call_NSDecimalNumber_initWithDecimal_,
