@@ -28,10 +28,17 @@ NS_ASSUME_NONNULL_BEGIN
 
 static int PyObjC_Initialized = 0;
 
+/*
+ * free-threaded: No lock needed becausethis value is set once
+ * during module init.
+ */
 PyObject* _Nullable PyObjCClass_DefaultModule;
 
 PyObject* _Nullable PyObjC_TypeStr2CFTypeID;
 
+#ifdef Py_GIL_DISABLED
+static PyMutex global_release_pool_lock = { 0 };
+#endif
 static NSAutoreleasePool* _Nullable global_release_pool;
 
 /* Calculate the current version of macOS in a format that
@@ -46,6 +53,10 @@ typedef struct {
 } NSOperatingSystemVersion;
 #endif
 
+/*
+ * free-threaded: No lock needed becausethis value is set once
+ * during module init.
+ */
 static NSOperatingSystemVersion gSystemVersion = {0, 0, 0};
 
 /*
@@ -58,7 +69,7 @@ static void
 calc_current_version(void)
 {
 #if PyObjC_BUILD_RELEASE >= 1010
-    if ([NSProcessInfo instancesRespondToSelector:@selector(operatingSystemVersion)]) {
+    if ([NSProcessInfo instancesRespondToSelector:@selector(operatingSystemVersion)]) { // LCOV_BR_EXCL_LINE
         NSAutoreleasePool* pool;
 
         pool           = [[NSAutoreleasePool alloc] init];
@@ -68,6 +79,7 @@ calc_current_version(void)
     } else
 #endif
     {
+        // LCOV_EXCL_START
         /* Code path for macOS 10.9 or earlier. Don't use Gestalt because that's
          * deprecated. */
         NSAutoreleasePool* pool;
@@ -98,7 +110,7 @@ calc_current_version(void)
         }
 
         [pool release];
-    }
+    } // LCOV_EXCL_STOP
 }
 
 PyObjC_FINAL_CLASS @interface OC_NSAutoreleasePoolCollector : NSObject
@@ -117,6 +129,7 @@ PyObjC_FINAL_CLASS @interface OC_NSAutoreleasePoolCollector : NSObject
 @implementation OC_NSAutoreleasePoolCollector
 + (void)newAutoreleasePool
 {
+    /* This method must be called with the mutex locked */
     OC_NSAutoreleasePoolCollector* value = [[self alloc] init];
     global_release_pool                  = [[NSAutoreleasePool alloc] init];
     (void)[value autorelease];
@@ -124,7 +137,13 @@ PyObjC_FINAL_CLASS @interface OC_NSAutoreleasePoolCollector : NSObject
 
 - (void)dealloc
 {
+#ifdef Py_GIL_DISABLED
+    PyMutex_Lock(&global_release_pool_lock);
+#endif
     global_release_pool = nil;
+#ifdef Py_GIL_DISABLED
+    PyMutex_Unlock(&global_release_pool_lock);
+#endif
     [super dealloc];
 }
 
@@ -195,22 +214,6 @@ static PyObject* _Nullable repythonify(PyObject* self __attribute__((__unused__)
     return rval;
 }
 
-static PyObject* _Nullable m_sizeoftype(PyObject* self __attribute__((__unused__)),
-                                        PyObject* value)
-{
-    if (!PyBytes_Check(value)) {
-        PyErr_SetString(PyExc_TypeError, "value must be a byte string");
-        return NULL;
-    }
-
-    Py_ssize_t size = PyObjCRT_SizeOfType(PyBytes_AsString(value));
-    if (size == -1) {
-        return NULL;
-    }
-
-    return PyLong_FromSsize_t(size);
-}
-
 PyDoc_STRVAR(macos_available_doc,
              "macos_available(major, minor, patch=0)\n" CLINIC_SEP "\n"
              "Return true if the current macOS release is "
@@ -300,21 +303,20 @@ static PyObject* _Nullable classAddMethods(PyObject* self __attribute__((__unuse
         return NULL;
     }
 
-    methodsArray = PySequence_Fast(methodsArray,
+    methodsArray = PyObjCSequence_Tuple(methodsArray,
                                    "Argument 'methodsArray' (pos 2) must be a sequence");
     if (methodsArray == NULL)
         return NULL;
 
-    int r = PyObjCClass_AddMethods(classObject, PySequence_Fast_ITEMS(methodsArray),
-                                   PySequence_Fast_GET_SIZE(methodsArray));
+    int r = PyObjCClass_AddMethods(classObject, PyTuple_ITEMS(methodsArray),
+                                   PyTuple_GET_SIZE(methodsArray));
     Py_DECREF(methodsArray);
 
     if (r == -1) {
         return NULL;
     }
 
-    Py_INCREF(Py_None);
-    return Py_None;
+    Py_RETURN_NONE;
 }
 
 PyDoc_STRVAR(have_autorelease_pool_doc,
@@ -323,9 +325,18 @@ PyDoc_STRVAR(have_autorelease_pool_doc,
 static PyObject*
 have_autorelease_pool(PyObject* self __attribute__((__unused__)))
 {
-    PyObject* result = global_release_pool ? Py_True : Py_False;
-    Py_INCREF(result);
-    return result;
+#ifdef Py_GIL_DISABLED
+    PyMutex_Lock(&global_release_pool_lock);
+#endif
+    bool have_pool = (global_release_pool != nil);
+#ifdef Py_GIL_DISABLED
+    PyMutex_Unlock(&global_release_pool_lock);
+#endif
+    if (have_pool) {
+        Py_RETURN_TRUE;
+    } else {
+        Py_RETURN_FALSE;
+    }
 }
 
 PyDoc_STRVAR(remove_autorelease_pool_doc,
@@ -343,8 +354,14 @@ static PyObject* _Nullable remove_autorelease_pool(PyObject* self
             /* Unconditionally clear the global autorelease pool,
              * there's not much we can do if releasing raises.
              */
+#ifdef Py_GIL_DISABLED
+            PyMutex_Lock(&global_release_pool_lock);
+#endif
             pool                = global_release_pool;
             global_release_pool = nil;
+#ifdef Py_GIL_DISABLED
+            PyMutex_Unlock(&global_release_pool_lock);
+#endif
             [pool release];
 
         } @catch (NSObject* localException) {
@@ -356,8 +373,7 @@ static PyObject* _Nullable remove_autorelease_pool(PyObject* self
     if (PyErr_Occurred())
         return NULL;
 
-    Py_INCREF(Py_None);
-    return Py_None;
+    Py_RETURN_NONE;
 }
 
 PyDoc_STRVAR(recycle_autorelease_pool_doc,
@@ -374,8 +390,14 @@ static PyObject* _Nullable recycle_autorelease_pool(PyObject* self
              * before calling release. There's not much we can
              * do if draining fails with an exception.
              */
+#ifdef Py_GIL_DISABLED
+    PyMutex_Lock(&global_release_pool_lock);
+#endif
             pool                = global_release_pool;
             global_release_pool = nil;
+#ifdef Py_GIL_DISABLED
+    PyMutex_Unlock(&global_release_pool_lock);
+#endif
             [pool release];
 
         } @catch (NSObject* localException) {
@@ -391,8 +413,7 @@ static PyObject* _Nullable recycle_autorelease_pool(PyObject* self
     if (PyErr_Occurred())
         return NULL;
 
-    Py_INCREF(Py_None);
-    return Py_None;
+    Py_RETURN_NONE;
 }
 
 PyDoc_STRVAR(getClassList_doc,
@@ -511,8 +532,8 @@ static PyObject* _Nullable loadBundle(PyObject* self __attribute__((__unused__))
     }
 
     class_list = PyObjC_GetClassList(1);
-    if (class_list == NULL) {
-        return NULL;
+    if (class_list == NULL) { // LCOV_BR_EXCL_LINE
+        return NULL; // LCOV_EXCL_LINE
     }
 
     len = PyTuple_GET_SIZE(class_list);
@@ -521,9 +542,6 @@ static PyObject* _Nullable loadBundle(PyObject* self __attribute__((__unused__))
         const char* nm;
 
         item = PyTuple_GET_ITEM(class_list, i);
-        if (item == NULL) {
-            continue;
-        }
 
         nm = ((PyTypeObject*)item)->tp_name;
 
@@ -532,12 +550,26 @@ static PyObject* _Nullable loadBundle(PyObject* self __attribute__((__unused__))
         } else if (strcmp(nm, "Object") == 0 || strcmp(nm, "List") == 0
                    || strcmp(nm, "Protocol") == 0) {
             /* skip, these have been deprecated since OpenStep! */
-        } else if (PyDict_SetItemString(module_globals, ((PyTypeObject*)item)->tp_name,
+        } else {
+            PyObject* py_nm = PyUnicode_FromString(nm);
+            if (py_nm == NULL) { // LCOV_BR_EXCL_LINE
+                // LCOV_EXCL_START
+                Py_DECREF(class_list);
+                class_list = NULL;
+                return NULL;
+                // LCOV_EXCL_STOP
+            }
+            if (PyDict_SetItem(module_globals, py_nm, // LCOV_BR_EXCL_LINE
                                         item)
-                   == -1) {
-            Py_DECREF(class_list);
-            class_list = NULL;
-            return NULL;
+                       == -1) {
+                // LCOV_EXCL_START
+                Py_DECREF(class_list);
+                Py_DECREF(py_nm);
+                class_list = NULL;
+                return NULL;
+                // LCOV_EXCL_STOP
+            }
+            Py_DECREF(py_nm);
         }
     }
     Py_XDECREF(class_list);
@@ -637,17 +669,17 @@ static PyObject* _Nullable objc_splitStructSignature(PyObject* self
         Py_INCREF(structname);
 
     } else {
-        structname = PyUnicode_FromStringAndSize(signature, end - signature - 1);
-        if (structname == NULL) {
+        if (end - signature - 1 < 0) {
+            PyErr_SetString(PyExc_ValueError, "value is not a complete struct signature");
             return NULL;
         }
+        structname = PyUnicode_FromStringAndSize(signature, end - signature - (end[-1]=='='));
+        if (structname == NULL) { // LCOV_BR_EXCL_LINE
+            return NULL; // LCOV_EXCL_LINE
+        }
     }
 
-    if (*end == '=') {
-        signature = end + 1;
-    } else {
-        signature = end;
-    }
+    signature = end;
 
     fields = PyList_New(0);
     if (fields == NULL) // LCOV_BR_EXCL_LINE
@@ -695,6 +727,7 @@ static PyObject* _Nullable objc_splitStructSignature(PyObject* self
         }
         t++;
 
+
         str = PyBytes_FromStringAndSize(signature, t - signature);
         if (str == NULL) { // LCOV_BR_EXCL_LINE
             // LCOV_EXCL_START
@@ -718,8 +751,8 @@ static PyObject* _Nullable objc_splitStructSignature(PyObject* self
             Py_DECREF(fields);
             Py_DECREF(item);
             Py_DECREF(structname);
-            // LCOV_EXCL_STOP
             return NULL;
+            // LCOV_EXCL_STOP
         }
         Py_DECREF(item);
 
@@ -787,12 +820,11 @@ static PyObject* _Nullable protocolsForProcess(PyObject* self __attribute__((__u
     unsigned int i;
 
     protlist = objc_copyProtocolList(&protCount);
-    if (protlist == NULL) {
-        Py_INCREF(Py_None);
-        return Py_None;
+    if (protlist == NULL) { // LCOV_BR_EXXCL_LINE
+        Py_RETURN_NONE; // LCOV_EXCL_LINE
     }
 
-    protocols = PyList_New(protCount);
+    protocols = PyList_New(0);
 
     if (protocols == NULL) { // LCOV_BR_EXCL_LINE
         return NULL;         // LCOV_EXCL_LINE
@@ -808,7 +840,14 @@ static PyObject* _Nullable protocolsForProcess(PyObject* self __attribute__((__u
             // LCOV_EXCL_STOP
         }
 
-        PyList_SET_ITEM(protocols, i, p);
+        int r = PyList_Append(protocols, p);
+        if (r == -1) { // LCOV_BR_EXCL_START
+            // LCOV_EXCL_START
+            Py_DECREF(protocols);
+            free(protlist);
+            return NULL;
+            // LCOV_EXCL_STOP
+        }
     }
 
     free(protlist);
@@ -823,32 +862,6 @@ static PyObject* _Nullable idSignatures(PyObject* self __attribute__((__unused__
     return PyObjCPointer_GetIDEncodings();
 }
 
-PyDoc_STRVAR(protocolNamed_doc,
-             "_protocolNamed(name)\n" CLINIC_SEP "\n"
-             "Returns an Objective-C protocol named *name*.\n"
-             "Raises AttributeError when no such protocol can be found.\n");
-static PyObject* _Nullable protocolNamed(PyObject* self __attribute__((__unused__)),
-                                         PyObject* _Nullable args,
-                                         PyObject* _Nullable kwds)
-{
-    static char* keywords[] = {"name", NULL};
-    char*        name;
-    Protocol*    p;
-
-    if (!PyArg_ParseTupleAndKeywords(args, kwds, "s", keywords, &name)) {
-        return NULL;
-    }
-
-    p = objc_getProtocol(name);
-
-    if (p == NULL) {
-        PyErr_SetString(PyExc_AttributeError, name);
-        return NULL;
-    }
-
-    return PyObjCFormalProtocol_ForProtocol(p);
-}
-
 PyDoc_STRVAR(protocolsForClass_doc,
              "protocolsForClass(cls)\n" CLINIC_SEP "\n"
              "Returns a list of Protocol objects that the class claims\n"
@@ -959,8 +972,7 @@ static PyObject* _Nullable registerMetaData(PyObject* self __attribute__((__unus
 
     } else {
         PyObjC_MappingCount++;
-        Py_INCREF(Py_None);
-        return Py_None;
+        Py_RETURN_NONE;
     }
 }
 
@@ -1044,21 +1056,21 @@ static PyObject* _Nullable createStructType(PyObject* self __attribute__((__unus
 
     if (pyfieldnames != Py_None) {
         pyfieldnames =
-            PySequence_Fast(pyfieldnames, "fieldnames must be a sequence of strings");
+            PyObjCSequence_Tuple(pyfieldnames, "fieldnames must be a sequence of strings");
 
         if (pyfieldnames == NULL)
             goto error_cleanup;
 
-        fieldnames = PyMem_Malloc(sizeof(char*) * PySequence_Fast_GET_SIZE(pyfieldnames));
+        fieldnames = PyMem_Malloc(sizeof(char*) * PyTuple_GET_SIZE(pyfieldnames));
         if (fieldnames == NULL) { // LCOV_BR_EXCL_LINE
             // LCOV_EXCL_START
             PyErr_NoMemory();
             goto error_cleanup;
             // LCOV_EXCL_STOP
         }
-        memset(fieldnames, 0, sizeof(char*) * PySequence_Fast_GET_SIZE(pyfieldnames));
-        for (i = 0; i < PySequence_Fast_GET_SIZE(pyfieldnames); i++) {
-            PyObject* v = PySequence_Fast_GET_ITEM(pyfieldnames, i);
+        memset(fieldnames, 0, sizeof(char*) * PyTuple_GET_SIZE(pyfieldnames));
+        for (i = 0; i < PyTuple_GET_SIZE(pyfieldnames); i++) {
+            PyObject* v = PyTuple_GET_ITEM(pyfieldnames, i);
             if (PyUnicode_Check(v)) {
                 PyObject* bytes = PyUnicode_AsEncodedString(v, NULL, NULL);
                 if (bytes == NULL) {
@@ -1078,7 +1090,7 @@ static PyObject* _Nullable createStructType(PyObject* self __attribute__((__unus
                 goto error_cleanup;
             }
         }
-        field_count = PySequence_Fast_GET_SIZE(pyfieldnames);
+        field_count = PyTuple_GET_SIZE(pyfieldnames);
 
     } else {
         field_count = -1;
@@ -1102,7 +1114,7 @@ error_cleanup:
         PyMem_Free(docstr);
 
     if (fieldnames) {
-        for (i = 0; i < PySequence_Fast_GET_SIZE(pyfieldnames); i++) {
+        for (i = 0; i < PyTuple_GET_SIZE(pyfieldnames); i++) {
             if (fieldnames[i])
                 PyMem_Free(fieldnames[i]);
         }
@@ -1169,8 +1181,14 @@ static PyObject* _Nullable registerCFSignature(PyObject* self __attribute__((__u
             return NULL; // LCOV_EXCL_LINE
         }
 
-        int r = PyDict_SetItemString(PyObjC_TypeStr2CFTypeID, encoding, v);
+        PyObject* py_encoding = PyUnicode_FromString(encoding);
+        if (py_encoding == NULL) {   // LCOV_BR_EXCL_LINE
+            return NULL; // LCOV_EXCL_LINE
+        }
+
+        int r = PyDict_SetItem(PyObjC_TypeStr2CFTypeID, py_encoding, v);
         Py_DECREF(v);
+        Py_DECREF(py_encoding);
         if (r == -1) {   // LCOV_BR_EXCL_LINE
             return NULL; // LCOV_EXCL_LINE
         }
@@ -1220,8 +1238,7 @@ static PyObject* _Nullable _updatingMetadata(PyObject* self __attribute__((__unu
         PyObjC_UpdatingMetaData = NO;
     }
 
-    Py_INCREF(Py_None);
-    return Py_None;
+    Py_RETURN_NONE;
 }
 
 /* Support for locking */
@@ -1242,8 +1259,7 @@ static PyObject* _Nullable PyObjC_objc_sync_enter(PyObject* self
     Py_END_ALLOW_THREADS
 
     if (rv == OBJC_SYNC_SUCCESS) {
-        Py_INCREF(Py_None);
-        return Py_None;
+        Py_RETURN_NONE;
     }
 
     PyErr_Format(PyObjCExc_LockError, "objc_sync_enter failed: %d", rv);
@@ -1265,8 +1281,7 @@ static PyObject* _Nullable PyObjC_objc_sync_exit(PyObject* self
         rv = objc_sync_exit(object);
     Py_END_ALLOW_THREADS
     if (rv == OBJC_SYNC_SUCCESS) {
-        Py_INCREF(Py_None);
-        return Py_None;
+        Py_RETURN_NONE;
     }
 
     PyErr_Format(PyObjCExc_LockError, "objc_sync_exit failed: %d", rv);
@@ -1337,39 +1352,46 @@ static PyObject* _Nullable _makeClosure(PyObject* self __attribute__((__unused__
         if (argIndex == -1) {
             PyErr_SetString(PyExc_ValueError,
                             "No callback argument in the specified object");
+            Py_CLEAR(methinfo);
             return NULL;
         }
 
     } else {
         if (argIndex < 0 || argIndex >= Py_SIZE(methinfo)) {
             PyErr_SetString(PyExc_IndexError, "No such argument");
+            Py_CLEAR(methinfo);
             return NULL;
         }
 
         if (methinfo->argtype[argIndex]->callable == NULL) {
             PyErr_Format(PyExc_ValueError,
                          "Argument %" PY_FORMAT_SIZE_T "d is not callable", argIndex);
+            Py_CLEAR(methinfo);
             return NULL;
         }
     }
 
-    PyObjC_callback_function result;
+    PyObjC_callback_function closure_func;
 
-    result =
+    closure_func =
         PyObjCFFI_MakeFunctionClosure(methinfo->argtype[argIndex]->callable, callable);
-    if (result == NULL) {
+    if (closure_func == NULL) {
+        Py_CLEAR(methinfo);
         return NULL;
     }
 
-    PyObject* retval = PyCapsule_New((void*)result, "objc.__imp__", _callback_cleanup);
-    if (retval == NULL) {
-        PyObjCFFI_FreeIMP((IMP)result);
+    PyObject* capsule = PyCapsule_New((void*)closure_func, "objc.__imp__", _callback_cleanup);
+    if (capsule == NULL) {
+        PyObjCFFI_FreeIMP((IMP)closure_func);
+        Py_CLEAR(methinfo);
         return NULL;
     }
 
-    return Py_BuildValue(
-        "NN", retval,
+    PyObject* result = Py_BuildValue(
+        "NN", capsule,
         PyObjCMethodSignature_AsDict(methinfo->argtype[argIndex]->callable));
+    Py_CLEAR(methinfo);
+    return result;
 }
 
 PyDoc_STRVAR(_closurePointer_doc, "_closurePointer(closure)\n" CLINIC_SEP "\n"
@@ -1437,8 +1459,7 @@ static PyObject* _Nullable PyObjC_setAssociatedObject(PyObject* self
     if (PyErr_Occurred())
         return NULL;
 
-    Py_INCREF(Py_None);
-    return Py_None;
+    Py_RETURN_NONE;
 }
 
 PyDoc_STRVAR(PyObjC_getAssociatedObject_doc,
@@ -1509,8 +1530,7 @@ static PyObject* _Nullable PyObjC_removeAssociatedObjects(PyObject* self
     if (PyErr_Occurred())
         return NULL;
 
-    Py_INCREF(Py_None);
-    return Py_None;
+    Py_RETURN_NONE;
 }
 
 static PyObject* _Nullable PyObjC_LoadConstant(PyObject* self __attribute__((__unused__)),
@@ -1552,36 +1572,6 @@ static PyObject* _Nullable PyObjC_LoadConstant(PyObject* self __attribute__((__u
     return v;
 }
 
-/* XXX: Move to utility file */
-PyObject* _Nullable PyObjC_callable_docstr_get(PyObject* callable, void* _Nullable closure
-                                               __attribute__((__unused__)))
-
-{
-    if (PyObjC_CallableDocFunction == NULL || PyObjC_CallableDocFunction == Py_None) {
-        Py_INCREF(Py_None);
-        return Py_None;
-    }
-    PyObject* args[2] = {NULL, callable};
-    return PyObject_Vectorcall(PyObjC_CallableDocFunction, args + 1,
-                               1 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
-}
-
-/* XXX: Move to utility file */
-PyObject* _Nullable PyObjC_callable_signature_get(PyObject* callable,
-                                                  void* _Nullable closure
-                                                  __attribute__((__unused__)))
-
-{
-    if (PyObjC_CallableSignatureFunction == NULL
-        || PyObjC_CallableSignatureFunction == Py_None) {
-        Py_INCREF(Py_None);
-        return Py_None;
-    }
-    PyObject* args[2] = {NULL, callable};
-    return PyObject_Vectorcall(PyObjC_CallableSignatureFunction, args + 1,
-                               1 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
-}
-
 static PyObject* _Nullable name_for_signature(PyObject* mod __attribute__((__unused__)),
                                               PyObject* signature)
 {
@@ -1600,8 +1590,7 @@ static PyObject* _Nullable name_for_signature(PyObject* mod __attribute__((__unu
             if (PyErr_Occurred()) { // LCOV_BR_EXCL_LINE
                 return NULL;        // LCOV_EXCL_LINE
             } else {
-                Py_INCREF(Py_None);
-                return Py_None;
+                Py_RETURN_NONE;
             }
         } else {
             return PyUnicode_FromString(type->tp_name);
@@ -1613,8 +1602,7 @@ static PyObject* _Nullable name_for_signature(PyObject* mod __attribute__((__unu
             return PyUnicode_FromString(name);
         }
     }
-    Py_INCREF(Py_None);
-    return Py_None;
+    Py_RETURN_NONE;
 }
 
 static PyObject* _Nullable block_signature(PyObject* mod __attribute__((__unused__)),
@@ -1625,10 +1613,14 @@ static PyObject* _Nullable block_signature(PyObject* mod __attribute__((__unused
         return NULL;
     }
 
-    const char* sig = PyObjCBlock_GetSignature(PyObjCObject_GetObject(block));
+    id objc_block = PyObjCObject_GetObject(block);
+    if (objc_block == nil) {
+        PyErr_SetString(PyObjCExc_Error, "Cannot get block signature of 'nil' block");
+        return NULL;
+    }
+    const char* sig = PyObjCBlock_GetSignature(objc_block);
     if (sig == NULL) {
-        Py_INCREF(Py_None);
-        return Py_None;
+        Py_RETURN_NONE;
     }
 
     return PyBytes_FromString(sig);
@@ -1660,8 +1652,7 @@ static PyObject* _Nullable force_rescan(PyObject* mod __attribute__((__unused__)
     }
 
 done:
-    Py_INCREF(Py_None);
-    return Py_None;
+    Py_RETURN_NONE;
 }
 
 #if PyObjC_BUILD_RELEASE >= 1100
@@ -1683,16 +1674,19 @@ static PyObject* _Nullable mod_dyld_shared_cache_contains_path(
         }
 
         int result = _dyld_shared_cache_contains_path(path);
-        return PyBool_FromLong(result);
+        if (result) {
+            Py_RETURN_TRUE;
+        } else {
+            Py_RETURN_FALSE;
+        }
     } else {
-        Py_INCREF(Py_False);
-        return Py_False;
+        Py_RETURN_FALSE;
     }
 }
 
 #else
 
-/* Variant to be used when buildin on macOS 10.15 or earlier:
+/* Variant to be used when building on macOS 10.15 or earlier:
  * use dlsym(3) APIs to look for the function.
  */
 
@@ -1701,10 +1695,21 @@ static PyObject* _Nullable mod_dyld_shared_cache_contains_path(
 {
     static bool (*contains_func)(const char*) = NULL;
     static bool resolved_func                 = 0;
+#ifdef Py_GIL_DISABLED
+    static PyMutex resolve_lock = { 0 };
+#endif
 
     if (!resolved_func) {
-        contains_func = dlsym(RTLD_DEFAULT, "_dyld_shared_cache_contains_path");
-        resolved_func = 1;
+#ifdef Py_GIL_DISABLED
+        PyMutex_Lock(&resolve_lock);
+        if (!resolved_func) {
+#endif
+            contains_func = dlsym(RTLD_DEFAULT, "_dyld_shared_cache_contains_path");
+            resolved_func = 1;
+#ifdef Py_GIL_DISABLED
+        }
+        PyMutex_Unlock(&resolve_lock);
+#endif
     }
 
     if (!PyUnicode_Check(object)) { // LCOV_BR_EXCL_LINE
@@ -1725,11 +1730,13 @@ static PyObject* _Nullable mod_dyld_shared_cache_contains_path(
 
     if (contains_func) {
         int result = contains_func(path);
-        return PyBool_FromLong(result);
+        if (result) {
+            Py_RETURN_TRUE;
+        } else {
+            Py_RETURN_FALSE;
+        }
     } else {
-
-        Py_INCREF(Py_False);
-        return Py_False;
+        Py_RETURN_FALSE;
     }
 }
 
@@ -1754,8 +1761,7 @@ static PyObject* _Nullable mod_registerVectorType(PyObject* _Nullable mod
         PyObjC_Assert(PyErr_Occurred(), NULL);
         return NULL;
     } else {
-        Py_INCREF(Py_None);
-        return Py_None;
+        Py_RETURN_NONE;
     }
 }
 
@@ -1788,8 +1794,7 @@ static PyObject* _Nullable mod_registeredMetadataForSelector(PyObject* _Nullable
     PyObjCMethodSignature* sig = PyObjCMethodSignature_GetRegistered(cls, sel);
     if (sig == NULL) {
         PyErr_Clear();
-        Py_INCREF(Py_None);
-        return Py_None;
+        Py_RETURN_NONE;
     }
     return PyObjCMethodSignature_AsDict(sig);
 }
@@ -1812,13 +1817,6 @@ static PyMethodDef mod_methods[] = {
         .ml_flags = METH_VARARGS | METH_KEYWORDS,
         .ml_doc   = objc_splitStructSignature_doc,
     },
-    {
-        .ml_name  = "_sizeOfType",
-        .ml_meth  = m_sizeoftype,
-        .ml_flags = METH_O,
-        .ml_doc   = "_sizeOfType(typestr, /)\n" CLINIC_SEP "\n"
-                    "Return the size of the type described by 'typestr'",
-    },
     {.ml_name  = "macos_available",
      .ml_meth  = (PyCFunction)macos_available,
      .ml_flags = METH_VARARGS | METH_KEYWORDS,
@@ -1871,10 +1869,6 @@ static PyMethodDef mod_methods[] = {
      .ml_meth  = (PyCFunction)protocolsForProcess,
      .ml_flags = METH_NOARGS,
      .ml_doc   = protocolsForProcess_doc},
-    {.ml_name  = "_protocolNamed",
-     .ml_meth  = (PyCFunction)protocolNamed,
-     .ml_flags = METH_VARARGS | METH_KEYWORDS,
-     .ml_doc   = protocolNamed_doc},
     {.ml_name  = "registerCFSignature",
      .ml_meth  = (PyCFunction)registerCFSignature,
      .ml_flags = METH_VARARGS | METH_KEYWORDS,
@@ -2142,7 +2136,9 @@ typedef int (*setup_function)(PyObject*);
  */
 static setup_function _Nullable setup_functions[] = {
     PyObjC_InitProxyRegistry, /* Must be first */
+    PyObjCClass_Setup,
 
+    PyObjCFFI_Setup,
     PyObjCUtil_Init,
     PyObjCPointerWrapper_Init,
     PyObjC_SetupOptions,
@@ -2183,11 +2179,11 @@ static setup_function _Nullable setup_functions[] = {
         for (; cur->name != NULL; cur++) {                                               \
             PyObject* t = item_creator(cur->value);                                      \
             if (t == NULL) { /* LCOV_BR_EXCL_LINE */                                     \
-                return NULL; /* LCOV_EXCL_LINE */                                        \
+                return -1; /* LCOV_EXCL_LINE */                                        \
             }                                                                            \
             if (PyModule_AddObject((module), cur->name, t)) { /* LCOV_BR_EXCL_LINE */    \
                 Py_DECREF(t);                                 /* LCOV_EXCL_LINE */       \
-                return NULL;                                  /* LCOV_EXCL_LINE */       \
+                return -1;                                  /* LCOV_EXCL_LINE */       \
             }                                                                            \
         }                                                                                \
     } while (0)
@@ -2198,31 +2194,23 @@ bytes_from_char(char ch)
     return PyBytes_FromStringAndSize(&ch, 1);
 }
 
-static struct PyModuleDef mod_module = {
-    PyModuleDef_HEAD_INIT, "_objc", NULL, 0, mod_methods, NULL, NULL, NULL, NULL};
 
-PyObject* _Nullable __attribute__((__visibility__("default"))) PyInit__objc(void)
+static int mod_exec_module(PyObject* m)
 {
     _Static_assert(sizeof(BOOL) == sizeof(bool), "BOOL and bool should have same size");
-    PyObject* m;
 
     if (PyObjC_Initialized) { // LCOV_BR_EXCL_LINE
         // LCOV_EXCL_START
         PyErr_SetString(PyExc_RuntimeError,
                         "Reload of objc._objc detected, this is not supported");
-        return NULL;
+        return -1;
         // LCOV_EXCL_STOP
     }
 
     calc_current_version();
 
-    m = PyModule_Create(&mod_module);
-    if (m == 0) {    // LCOV_BR_EXCL_LINE
-        return NULL; // LCOV_EXCL_LINE // LCOV_EXCL_LINE
-    }
-
     if (PyObjC_InitSuperCallRegistry() == -1) { // LCOV_BR_EXCL_LINE
-        return NULL;                            // LCOV_EXCL_LINE
+        return -1;                            // LCOV_EXCL_LINE
     }
 
     /* Create a temporary release pool for handling values autoreleased during
@@ -2235,63 +2223,63 @@ PyObject* _Nullable __attribute__((__visibility__("default"))) PyInit__objc(void
 
     PyObjCClass_DefaultModule = PyUnicode_FromString("objc");
     if (PyObjCClass_DefaultModule == NULL) { // LCOV_BR_EXCL_LINE
-        return NULL;                         // LCOV_EXCL_LINE
+        return -1;                         // LCOV_EXCL_LINE
     }
 
     PyObjC_TypeStr2CFTypeID = PyDict_New();
     if (PyObjC_TypeStr2CFTypeID == NULL) { // LCOV_BR_EXCL_LINE
-        return NULL;                       // LCOV_EXCL_LINE
+        return -1;                       // LCOV_EXCL_LINE
     }
 
     /* XXX: Move these to setup functions as well */
     if (PyType_Ready(&PyObjCMetaClass_Type) < 0) { // LCOV_BR_EXCL_LINE
-        return NULL;                               // LCOV_EXCL_LINE
+        return -1;                               // LCOV_EXCL_LINE
     }
     if (PyType_Ready(&PyObjCClass_Type) < 0) { // LCOV_BR_EXCL_LINE
-        return NULL;                           // LCOV_EXCL_LINE
+        return -1;                           // LCOV_EXCL_LINE
     }
     if (PyType_Ready((PyTypeObject*)&PyObjCObject_Type) < 0) { // LCOV_BR_EXCL_LINE
-        return NULL;                                           // LCOV_EXCL_LINE
+        return -1;                                           // LCOV_EXCL_LINE
     }
 
     if (PyType_Ready(&StructBase_Type) < 0) { // LCOV_BR_EXCL_LINE
-        return NULL;                          // LCOV_EXCL_LINE
+        return -1;                          // LCOV_EXCL_LINE
     }
 
     for (setup_function* cur = setup_functions; *cur != NULL; cur++) {
         if ((*cur)(m) < 0) { // LCOV_BR_EXCL_LINE
-            return NULL;     // LCOV_EXCL_LINE
+            return -1;     // LCOV_EXCL_LINE
         }
-        if (PyErr_Occurred()) {
-            return NULL;
+        if (PyErr_Occurred()) { // LCOV_BR_EXCL_LINE
+            return -1; // LCOV_EXCL_LINE
         }
     }
 
     /* XXX: Move these to setup functions as well */
-    if ( // LCOV_BR_EXCL_LINE
-        PyModule_AddObject(m, "objc_meta_class", (PyObject*)&PyObjCMetaClass_Type) < 0) {
-        return NULL; // LCOV_EXCL_LINE
+    if (PyModule_AddObject( // LCOV_BR_EXCL_LINE
+            m, "objc_meta_class", (PyObject*)&PyObjCMetaClass_Type) < 0) {
+        return -1; // LCOV_EXCL_LINE
     }
     Py_INCREF((PyObject*)&PyObjCMetaClass_Type);
 
     if (PyModule_AddObject( // LCOV_BR_EXCL_LINE
             m, "objc_class", (PyObject*)&PyObjCClass_Type)
         < 0) {
-        return NULL; // LCOV_EXCL_LINE
+        return -1; // LCOV_EXCL_LINE
     }
     Py_INCREF((PyObject*)&PyObjCClass_Type);
 
     if (PyModule_AddObject( // LCOV_BR_EXCL_LINE
             m, "objc_object", (PyObject*)&PyObjCObject_Type)
         < 0) {
-        return NULL; // LCOV_EXCL_LINE
+        return -1; // LCOV_EXCL_LINE
     }
     Py_INCREF((PyObject*)&PyObjCObject_Type);
 
     if (PyModule_AddObject( // LCOV_BR_EXCL_LINE
             m, "_structwrapper", (PyObject*)&StructBase_Type)
         < 0) {
-        return NULL; // LCOV_EXCL_LINE
+        return -1; // LCOV_EXCL_LINE
     }
     Py_INCREF((PyObject*)&StructBase_Type);
 
@@ -2313,8 +2301,14 @@ PyObject* _Nullable __attribute__((__visibility__("default"))) PyInit__objc(void
     /* Allocate an auto-release pool for our own use, this avoids numerous
      * warnings during startup of a python script.
      */
+#ifdef Py_GIL_DISABLED
+    PyMutex_Lock(&global_release_pool_lock);
+#endif
     global_release_pool = nil;
     [OC_NSAutoreleasePoolCollector newAutoreleasePool];
+#ifdef Py_GIL_DISABLED
+    PyMutex_Unlock(&global_release_pool_lock);
+#endif
 
     /*
      * Archives created with Python 2.x can contain instances of
@@ -2328,7 +2322,54 @@ PyObject* _Nullable __attribute__((__visibility__("default"))) PyInit__objc(void
 #pragma clang diagnostic pop
 
     PyObjC_Initialized = 1;
-    return m;
+    return 0;
 }
 
+static struct PyModuleDef_Slot mod_slots[] = {
+    {
+        .slot = Py_mod_exec,
+        .value = (void*)mod_exec_module
+    },
+#if PY_VERSION_HEX >= 0x030c0000
+    {
+        /* This extension does not use the CPython API other than initializing
+         * the module, hence is safe with subinterpreters and per-interpreter
+         * GILs
+         */
+        .slot = Py_mod_multiple_interpreters,
+        .value = Py_MOD_MULTIPLE_INTERPRETERS_NOT_SUPPORTED,
+    },
+#endif
+#if PY_VERSION_HEX >= 0x030d0000
+    {
+        .slot = Py_mod_gil,
+        .value = Py_MOD_GIL_NOT_USED,
+    },
+#endif
+    {  /* Sentinel */
+        .slot = 0,
+        .value = 0
+    }
+};
+
+static struct PyModuleDef mod_module = {
+    .m_base = PyModuleDef_HEAD_INIT,
+    .m_name = "_objc",
+    .m_doc = NULL,
+    .m_size = 0,
+    .m_methods = mod_methods,
+    .m_slots = mod_slots,
+    .m_traverse = NULL,
+    .m_clear = NULL,
+    .m_free = NULL,
+};
+
+PyObject* PyInit__objc(void);
+
+PyObject* __attribute__((__visibility__("default"))) _Nullable PyInit__objc(void)
+{
+    return PyModuleDef_Init(&mod_module);
+}
+
+
 NS_ASSUME_NONNULL_END
