@@ -23,10 +23,16 @@ PyAPI_FUNC(int) PyArg_Parse(PyObject*, const char*, ...)
     __attribute__((warn_unused_result));
 PyAPI_FUNC(int) PyArg_ParseTuple(PyObject* _Nullable, const char*, ...)
     __attribute__((warn_unused_result));
+#if PY_VERSION_HEX >= 0x030d0000
+PyAPI_FUNC(int)
+    PyArg_ParseTupleAndKeywords(PyObject* _Nullable, PyObject* _Nullable,
+                                const char* _Nullable, char* _Nullable const* _Nullable, ...)
+        __attribute__((warn_unused_result));
+#else
 PyAPI_FUNC(int)
     PyArg_ParseTupleAndKeywords(PyObject* _Nullable, PyObject* _Nullable,
                                 const char* _Nullable, char* _Nullable* _Nullable, ...)
-        __attribute__((warn_unused_result));
+#endif
 
 PyAPI_FUNC(PyObject* _Nullable) PyBool_FromLong(long) __attribute__((warn_unused_result));
 PyAPI_FUNC(void) PyBuffer_Release(Py_buffer* view);
@@ -71,15 +77,15 @@ PyAPI_FUNC(PyObject* _Nullable) PyDict_Copy(PyObject* mp)
 PyAPI_FUNC(int) PyDict_DelItem(PyObject* mp, PyObject* key)
     __attribute__((warn_unused_result));
 PyAPI_FUNC(int) PyDict_DelItemString(PyObject* dp, const char* key)
-    __attribute__((warn_unused_result));
+    __attribute__((warn_unused_result)) __attribute__((unavailable));
 PyAPI_FUNC(PyObject* _Nullable) PyDict_GetItem(PyObject* mp, PyObject* key)
     __attribute__((warn_unused_result));
 PyAPI_FUNC(PyObject* _Nullable) PyDict_GetItemString(PyObject* dp, const char* key)
-    __attribute__((warn_unused_result));
+    __attribute__((warn_unused_result)) __attribute__((unavailable));
 PyAPI_FUNC(PyObject* _Nullable) _PyDict_GetItemStringWithError(PyObject*, const char*)
-    __attribute__((warn_unused_result));
+    __attribute__((warn_unused_result)) __attribute__((unavailable));
 PyAPI_FUNC(PyObject* _Nullable) PyDict_GetItemWithError(PyObject* mp, PyObject* key)
-    __attribute__((warn_unused_result));
+    __attribute__((warn_unused_result)) __attribute__((unavailable));
 PyAPI_FUNC(PyObject* _Nullable) PyDict_Keys(PyObject* mp)
     __attribute__((warn_unused_result));
 PyAPI_FUNC(PyObject* _Nullable) PyDict_New(void) __attribute__((warn_unused_result));
@@ -89,7 +95,7 @@ PyAPI_FUNC(int) PyDict_Next(PyObject* p, Py_ssize_t* ppos, PyObject* _Nullable*
 PyAPI_FUNC(int) PyDict_SetItem(PyObject* mp, PyObject* key, PyObject* item)
     __attribute__((warn_unused_result));
 PyAPI_FUNC(int) PyDict_SetItemString(PyObject* dp, const char* key, PyObject* item)
-    __attribute__((warn_unused_result));
+    __attribute__((warn_unused_result)) __attribute__((unavailable));
 PyAPI_FUNC(Py_ssize_t) PyDict_Size(PyObject* mp) __attribute__((warn_unused_result));
 PyAPI_FUNC(int) PyDict_Update(PyObject* mp, PyObject* other)
     __attribute__((warn_unused_result));
@@ -145,7 +151,7 @@ PyAPI_FUNC(int) PyList_Append(PyObject*, PyObject*) __attribute__((warn_unused_r
 PyAPI_FUNC(PyObject* _Nullable) PyList_AsTuple(PyObject*)
     __attribute__((warn_unused_result));
 PyAPI_FUNC(PyObject* _Nullable) PyList_GetItem(PyObject*, Py_ssize_t)
-    __attribute__((warn_unused_result));
+    __attribute__((warn_unused_result)) __attribute__((unavailable));
 PyAPI_FUNC(PyObject* _Nullable) PyList_New(Py_ssize_t size)
     __attribute__((warn_unused_result));
 PyAPI_FUNC(Py_ssize_t) PyList_Size(PyObject*) __attribute__((warn_unused_result));
@@ -197,7 +203,7 @@ PyAPI_FUNC(int) PyModule_AddObject(PyObject* mod, const char*, PyObject* value)
 PyAPI_FUNC(int) PyModule_AddStringConstant(PyObject*, const char*, const char*)
     __attribute__((warn_unused_result));
 PyAPI_FUNC(PyObject* _Nullable) PyModule_GetDict(PyObject*)
-    __attribute__((warn_unused_result));
+    __attribute__((warn_unused_result)) __attribute__((__unavailable__));
 PyAPI_FUNC(Py_ssize_t) PyNumber_AsSsize_t(PyObject* o, PyObject* _Nullable exc)
     __attribute__((warn_unused_result));
 PyAPI_FUNC(PyObject* _Nullable) PyNumber_Float(PyObject* o)
@@ -266,7 +272,7 @@ PyAPI_FUNC(int) PySequence_Contains(PyObject* seq, PyObject* ob)
 PyAPI_FUNC(int) PySequence_DelItem(PyObject* o, Py_ssize_t i)
     __attribute__((warn_unused_result));
 PyAPI_FUNC(PyObject* _Nullable) PySequence_Fast(PyObject* o, const char* m)
-    __attribute__((warn_unused_result));
+    __attribute__((warn_unused_result)) __attribute__((unavailable));
 PyAPI_FUNC(PyObject* _Nullable) PySequence_GetItem(PyObject* o, Py_ssize_t i)
     __attribute__((warn_unused_result));
 PyAPI_FUNC(Py_ssize_t) PySequence_Length(PyObject* o) __attribute__((warn_unused_result));
@@ -384,7 +390,7 @@ PyAPI_FUNC(int) PyBuffer_FillInfo(Py_buffer* view, PyObject* _Nullable exporter,
     __attribute__((warn_unused_result));
 PyAPI_FUNC(void) PyBuffer_Release(Py_buffer* view);
 PyAPI_FUNC(const Py_buffer*) PyPickleBuffer_GetBuffer(PyObject*)
-    __attribute__((warn_unused_result));
+    __attribute__((warn_unused_result)) __attribute__((unavailable));
 PyAPI_FUNC(PyObject* _Nullable) PyMemoryView_FromBuffer(const Py_buffer* info)
     __attribute__((warn_unused_result));
 PyAPI_FUNC(int) PySlice_Unpack(PyObject* slice, Py_ssize_t* start, Py_ssize_t* stop,
@@ -393,5 +399,16 @@ PyAPI_FUNC(Py_ssize_t) PySlice_AdjustIndices(Py_ssize_t length, Py_ssize_t* star
                                              Py_ssize_t* stop, Py_ssize_t step)
     __attribute__((warn_unused_result));
 
+#if PY_VERSION_HEX >= 0x030d0000
+
+PyAPI_FUNC(PyObject* _Nullable) PyList_GetItemRef(PyObject*, Py_ssize_t)
+    __attribute__((warn_unused_result));
+PyAPI_FUNC(int) PyDict_GetItemRef(PyObject *p, PyObject *key, PyObject * _Nonnull* _Nullable result) __attribute__((warn_unused_result));
+
+#endif
+
+static inline Py_ALWAYS_INLINE  void (Py_INCREF)(PyObject * _Nonnull op);
+
+
 NS_ASSUME_NONNULL_END
 #endif /* USE_STATIC_ANALYZER */
