@@ -8,6 +8,7 @@ import pickle
 import sys
 import test.pickletester
 import collections
+import pathlib
 
 import objc
 import objc._pycoder as pycoder
@@ -24,6 +25,7 @@ from PyObjCTest.fnd import (
     NSSet,
     NSString,
     NSUnarchiver,
+    NSURL,
 )
 from PyObjCTools.TestSupport import (
     TestCase,
@@ -241,6 +243,34 @@ class TestKeyedArchiveSimple(TestCase):
         self.archiverClass = NSKeyedArchiver
         self.unarchiverClass = NSKeyedUnarchiver
 
+    def test_without_helper(self):
+        orig = objc.options._nscoding_decoder
+        try:
+            objc.options._nscoding_decoder = None
+
+            p = object()
+
+            buf = self.archiverClass.archivedDataWithRootObject_(p)
+            self.assertIsInstance(buf, NSData)
+
+            with self.assertRaisesRegex(
+                ValueError, "decoding Python objects is not supported"
+            ):
+                self.unarchiverClass.unarchiveObjectWithData_(buf)
+
+        finally:
+            objc.options._nscoding_decoder = orig
+
+    def test_roundtrip_pathlib_path(self):
+        p = pathlib.Path(__file__)
+
+        buf = self.archiverClass.archivedDataWithRootObject_(p)
+        self.assertIsInstance(buf, NSData)
+        v = self.unarchiverClass.unarchiveObjectWithData_(buf)
+        self.assertNotIsInstance(v, pathlib.Path)
+        self.assertIsInstance(v, NSURL)
+        self.assertEqual(v, p)
+
     def test_global_from_main(self):
         import __main__ as mod
 
@@ -315,6 +345,13 @@ class TestKeyedArchiveSimple(TestCase):
         if self.archiverClass is NSKeyedArchiver:
             archiver.finishEncoding()
 
+    def test_local_function(self):
+        def function():
+            pass
+
+        with self.assertRaises(pickle.PicklingError):
+            self.archiverClass.archivedDataWithRootObject_(function)
+
     def test_various_objects(self):
         o = a_newstyle_class()
         o.attr1 = False
@@ -466,6 +503,44 @@ class TestKeyedArchiveSimple(TestCase):
         if self.archiverClass is NSKeyedArchiver:
             archiver.finishEncoding()
 
+        data = NSMutableData.alloc().init()
+        archiver = self.archiverClass.alloc().initForWritingWithMutableData_(data)
+        with self.assertRaises(pickle.PicklingError):
+            archiver.encodeRootObject_([1, 2, invalid_reduce()])
+        if self.archiverClass is NSKeyedArchiver:
+            archiver.finishEncoding()
+
+        data = NSMutableData.alloc().init()
+        archiver = self.archiverClass.alloc().initForWritingWithMutableData_(data)
+        with self.assertRaises(pickle.PicklingError):
+            archiver.encodeRootObject_({1, 2, invalid_reduce()})
+        if self.archiverClass is NSKeyedArchiver:
+            archiver.finishEncoding()
+
+        data = NSMutableData.alloc().init()
+        archiver = self.archiverClass.alloc().initForWritingWithMutableData_(data)
+        with self.assertRaises(pickle.PicklingError):
+            archiver.encodeRootObject_({1: 2, 2: invalid_reduce()})
+        if self.archiverClass is NSKeyedArchiver:
+            archiver.finishEncoding()
+
+        data = NSMutableData.alloc().init()
+        archiver = self.archiverClass.alloc().initForWritingWithMutableData_(data)
+        with self.assertRaises(pickle.PicklingError):
+            archiver.encodeRootObject_({1: 2, invalid_reduce(): 4})
+        if self.archiverClass is NSKeyedArchiver:
+            archiver.finishEncoding()
+
+        class List(list):
+            pass
+
+        data = NSMutableData.alloc().init()
+        archiver = self.archiverClass.alloc().initForWritingWithMutableData_(data)
+        with self.assertRaises(pickle.PicklingError):
+            archiver.encodeRootObject_(List([invalid_reduce()]))
+        if self.archiverClass is NSKeyedArchiver:
+            archiver.finishEncoding()
+
         class invalid_reduce:
             def __reduce__(self):
                 return (1,)
@@ -1840,7 +1915,7 @@ class TestSecureArchivingPython(TestCase):
         with pyobjc_options(_nscoding_encoder=None):
             with self.assertRaisesRegex(
                 ValueError,
-                "NSInvalidArgumentException - encoding Python objects is not supported",
+                "encoding Python objects is not supported",
             ):
                 NSKeyedArchiver.archivedDataWithRootObject_(object())
 
@@ -1849,6 +1924,6 @@ class TestSecureArchivingPython(TestCase):
         with pyobjc_options(_nscoding_decoder=None):
             with self.assertRaisesRegex(
                 ValueError,
-                "NSInvalidArgumentException - decoding Python objects is not supported",
+                "decoding Python objects is not supported",
             ):
                 NSKeyedUnarchiver.unarchiveObjectWithData_(buf)
