@@ -87,8 +87,11 @@ for method, argmeta in [
     (b"passOutFloat:", {2: {"type_modifier": b"o"}}),
     (b"passInFloat:", {2: {"type_modifier": b"n"}}),
     (b"passInOutDouble:", {2: {"type_modifier": b"N"}}),
+    (b"passInOutLongDouble:", {2: {"type_modifier": b"N"}}),
     (b"passOutDouble:", {2: {"type_modifier": b"o"}}),
+    (b"passOutLongDouble:", {2: {"type_modifier": b"o"}}),
     (b"passInDouble:", {2: {"type_modifier": b"n"}}),
+    (b"passInLongDouble:", {2: {"type_modifier": b"n"}}),
     (b"passInOutCharp:", {2: {"type_modifier": b"N"}}),
     (b"passOutCharp:", {2: {"type_modifier": b"o"}}),
     (b"passInCharp:", {2: {"type_modifier": b"n"}}),
@@ -195,6 +198,13 @@ class PyOCTestSimpleReturns(TestCase):
         self.assertEqual(OC_TestClass1.doubleClsFunc(), 42.0)
         self.assertEqual(OC_TestClass1.doubleClsFunc(), 1e10)
 
+    def testClsLongDouble(self):
+        OC_TestClass1.clsReset()
+        self.assertEqual(OC_TestClass1.longdoubleClsFunc(), 0.128)
+        self.assertEqual(OC_TestClass1.longdoubleClsFunc(), 1.0)
+        self.assertEqual(OC_TestClass1.longdoubleClsFunc(), 42.0)
+        self.assertEqual(OC_TestClass1.longdoubleClsFunc(), 1e10)
+
     def testClsCharp(self):
         OC_TestClass1.clsReset()
         self.assertEqual(OC_TestClass1.charpClsFunc(), b"hello")
@@ -306,6 +316,14 @@ class PyOCTestSimpleReturns(TestCase):
         self.assertEqual(obj.doubleFunc(), 42.0)
         self.assertEqual(obj.doubleFunc(), 1e10)
 
+    def testLongDouble(self):
+        obj = OC_TestClass1.new()
+        obj.reset()
+        self.assertEqual(obj.longdoubleFunc(), 0.128)
+        self.assertEqual(obj.longdoubleFunc(), 1.0)
+        self.assertEqual(obj.longdoubleFunc(), 42.0)
+        self.assertEqual(obj.longdoubleFunc(), 1e10)
+
     def testCharp(self):
         obj = OC_TestClass1.new()
         obj.reset()
@@ -785,6 +803,10 @@ class PyOCTestByReferenceArguments(TestCase):
         self.assertEqual(self.obj.passInDouble_(10), 90.0)
         self.assertEqual(self.obj.passInDouble_(0.11), 0.99)
 
+    def testLongDoubleIn(self):
+        self.assertEqual(self.obj.passInLongDouble_(10), 90.0)
+        self.assertEqual(self.obj.passInLongDouble_(0.11), 0.99)
+
     def testDoubleOut(self):
         self.obj.reset()
         self.assertEqual(self.obj.passOutDouble_(None), 0.128)
@@ -792,11 +814,23 @@ class PyOCTestByReferenceArguments(TestCase):
         self.assertEqual(self.obj.passOutDouble_(None), 42.0)
         self.assertEqual(self.obj.passOutDouble_(None), 1e10)
 
+    def testLongDoubleOut(self):
+        self.obj.reset()
+        self.assertEqual(self.obj.passOutLongDouble_(None), 0.128)
+        self.assertEqual(self.obj.passOutLongDouble_(None), 1.0)
+        self.assertEqual(self.obj.passOutLongDouble_(None), 42.0)
+        self.assertEqual(self.obj.passOutLongDouble_(None), 1e10)
+
     def testDoubleInOut(self):
         self.assertEqual(self.obj.passInOutDouble_(10), 420)
         self.assertEqual(self.obj.passInOutDouble_(10.0), 420)
         self.assertEqual(self.obj.passInOutDouble_(0.01), 0.42)
 
+    def testLongDoubleInOut(self):
+        self.assertEqual(self.obj.passInOutLongDouble_(10), 420)
+        self.assertEqual(self.obj.passInOutLongDouble_(10.0), 420)
+        self.assertEqual(self.obj.passInOutLongDouble_(0.01), 0.42)
+
     def testCharpIn(self):
         self.assertEqual(self.obj.passInCharp_(b"hello"), b"hheelllloo")
         self.assertEqual(self.obj.passInCharp_(b"abcde"), b"aabbccddee")
@@ -863,6 +897,7 @@ ULONGLONG_NUMBERS = [
 
 FLOAT_NUMBERS = [makeCFloat(0.1), makeCFloat(100.0)]
 DOUBLE_NUMBERS = [1.5, 3.5, 1e10, 1.99e10]
+LONG_DOUBLE_NUMBERS = [1.5, 3.5, 1e10, 1.99e10]
 OBJECTS = ["hello", 1.0, range(4), lambda x: 10]
 DUMMY_OBJECTS = [(1, 1), (-10, -10), (-4, -5), (0, 0), (10, 20)]
 DUMMY2_OBJECTS = [((1, 2, 3, 4),), ((-9, -8, -7, -6),)]
@@ -977,6 +1012,15 @@ class MyOCClass(objc.lookUpClass("NSObject")):
 
     doubleFunc = objc.selector(doubleFunc, signature=OC_TestClass1.doubleFunc.signature)
 
+    def longdoubleFunc(self):
+        i = self.idx
+        self.idx += 1
+        return LONG_DOUBLE_NUMBERS[i]
+
+    longdoubleFunc = objc.selector(
+        longdoubleFunc, signature=OC_TestClass1.longdoubleFunc.signature
+    )
+
     def idFunc(self):
         i = self.idx
         self.idx += 1
@@ -1069,6 +1113,13 @@ class MyOCClass(objc.lookUpClass("NSObject")):
 
     doubleArg_ = objc.selector(doubleArg_, signature=OC_TestClass1.doubleArg_.signature)
 
+    def longdoubleArg_(self, arg):
+        return arg * 2
+
+    longdoubleArg_ = objc.selector(
+        longdoubleArg_, signature=OC_TestClass1.longdoubleArg_.signature
+    )
+
 
 class OCPyTestSimpleCalls(TestCase):
     #
@@ -1510,6 +1561,13 @@ class OCPyTestSimpleCalls(TestCase):
         for o in DOUBLE_NUMBERS:
             self.assertEqual(self.obj.callInstanceDoubleFuncOf_(self.ocobj), o)
 
+    def testLongCDouble(self):
+        self.pyobj.reset()
+        self.ocobj.reset()
+
+        for o in LONG_DOUBLE_NUMBERS:
+            self.assertEqual(self.obj.callInstanceLongDoubleFuncOf_(self.ocobj), o)
+
     def testIDouble(self):
         self.pyobj.reset()
         self.ocobj.reset()
@@ -1517,6 +1575,13 @@ class OCPyTestSimpleCalls(TestCase):
         for o in DOUBLE_NUMBERS:
             self.assertEqual(self.obj.invokeInstanceDoubleFuncOf_(self.ocobj), o)
 
+    def testILongDouble(self):
+        self.pyobj.reset()
+        self.ocobj.reset()
+
+        for o in LONG_DOUBLE_NUMBERS:
+            self.assertEqual(self.obj.invokeInstanceLongDoubleFuncOf_(self.ocobj), o)
+
     def testCId(self):
         self.pyobj.reset()
         self.ocobj.reset()
