@@ -60,9 +60,7 @@ class TestPythonProxy(TestCase):
 
     def test_copy_without_helper(self):
         with pyobjc_options(_copy=None):
-            with self.assertRaisesRegex(
-                ValueError, "NSInvalidArgumentException - cannot copy Python objects"
-            ):
+            with self.assertRaisesRegex(ValueError, "cannot copy Python objects"):
                 OC_ObjectInt.copyObject_withZone_(self.value, None)
 
     def test_description(self):
@@ -180,27 +178,6 @@ class TestPlainPythonMethods(TestCase):
         ):
             OC_ObjectInt.nosuchSelectorOf_(forwarder)
 
-    def test_too_long_selector(self):
-        selector = "a" * 300
-
-        class Subclass(Forwarder):
-            pass
-
-        def meth(self):
-            return 42
-
-        meth.__name__ = selector
-
-        setattr(Subclass, selector, meth)
-
-        value = Subclass()
-        self.assertEqual(getattr(value, selector)(), 42)
-
-        with self.assertRaisesRegex(
-            ValueError, "NSInvalidArgumentException - <.*> does not recognize -.*"
-        ):
-            OC_ObjectInt.invokeSelector_of_(selector.encode(), value)
-
     def test_not_method(self):
         value = Forwarder()
         value.attr = 42
